@@ -3,7 +3,7 @@
  *
  * This file is part of msmtp, an SMTP client.
  *
- * Copyright (C) 2000, 2003, 2004, 2005
+ * Copyright (C) 2000, 2003, 2004, 2005, 2006
  * Martin Lambers <marlam@marlam.de>
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -22,7 +22,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+# include "config.h"
 #endif
 
 #include <stdio.h>
@@ -36,23 +36,22 @@ extern int errno;
 #include <getopt.h>
 extern char *optarg;
 extern int optind;
-#ifdef ENABLE_NLS
-#include <locale.h>
-#endif
+#include <unistd.h>
 #include <sysexits.h>
-#ifndef EX_OK
-#define EX_OK 0
+#ifdef ENABLE_NLS
+# include <locale.h>
 #endif
 #ifdef HAVE_SYSLOG
-#include <syslog.h>
+# include <syslog.h>
 #endif
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-#ifdef __MINGW32__
+#ifdef W32_NATIVE
+#include <io.h>
+#include <fcntl.h>
 #include <windows.h>
 #include <winsock2.h>
 #elif defined DJGPP
+#include <io.h>
+#include <fcntl.h>
 #include <netdb.h>
 #include <arpa/inet.h>
 #else /* UNIX */
@@ -60,24 +59,26 @@ extern int optind;
 #include <arpa/inet.h>
 #endif
 
+#include "getpass.h"
 #include "gettext.h"
 #include "xalloc.h"
 #include "xvasprintf.h"
 
-#include "list.h"
-#include "os_env.h"
 #include "conf.h"
+#include "list.h"
 #include "net.h"
 #include "netrc.h"
 #include "smtp.h"
-#ifdef HAVE_SSL
-#include "tls.h"
-#endif /* HAVE_SSL */
+#include "tools.h"
+#ifdef HAVE_TLS
+# include "tls.h"
+#endif /* HAVE_TLS */
 
 /* Default file names. */
-#ifdef __MINGW32__
+#ifdef W32_NATIVE
 #define SYSCONFFILE	"msmtprc.txt"
 #define USERCONFFILE	"msmtprc.txt"
+#define NETRCFILE	"netrc.txt"
 #elif defined (DJGPP)
 #define SYSCONFFILE	"msmtprc"
 #define USERCONFFILE	"_msmtprc"
@@ -98,7 +99,7 @@ const char *prgname;
 
 void xalloc_die(void)
 {
-    fprintf(stderr, "%s: FATAL: %s", prgname, strerror(errno));
+    fprintf(stderr, "%s: FATAL: %s", prgname, strerror(ENOMEM));
     exit(EX_OSERR);
 }
 
@@ -130,7 +131,7 @@ int exitcode_net(int net_error_code)
     }
 }
 
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
 int exitcode_tls(int tls_error_code)
 {
     switch (tls_error_code)
@@ -154,7 +155,7 @@ int exitcode_tls(int tls_error_code)
 	    return EX_SOFTWARE;
     }
 }
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
 
 int exitcode_smtp(int smtp_error_code)
 {
@@ -278,11 +279,6 @@ char *msmtp_sanitize_string(char *str)
 
 char *msmtp_password_callback(const char *hostname, const char *user)
 {
-#ifdef __MINGW32__
-    /* Windows has neither a .netrc nor a getpass() substitute ... */
-    return NULL;
-#else
-	
     char *homedir;
     char *netrc_filename;
     netrc_entry *netrc_hostlist;
@@ -316,8 +312,6 @@ char *msmtp_password_callback(const char *hostname, const char *user)
     }
     
     return password;
-
-#endif
 }
 
 
@@ -327,7 +321,7 @@ char *msmtp_password_callback(const char *hostname, const char *user)
  * Prints information about a TLS certificate.
  */
 
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
 /* Convert the given time into a string. */
 void msmtp_time_to_string(time_t *t, char *buf, size_t bufsize)
 {
@@ -445,9 +439,9 @@ int msmtp_rmqs(account_t *acc, int debug, const char *rmqs_argument,
 {
     smtp_server_t srv;
     int e;
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
     tls_cert_info_t *tci = NULL;
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
     
     *errstr = NULL;
     *msg = NULL;
@@ -455,18 +449,6 @@ int msmtp_rmqs(account_t *acc, int debug, const char *rmqs_argument,
     /* create a new smtp_server_t */
     srv = smtp_new(debug ? stdout : NULL, acc->protocol);
 
-    /* prepare tls */
-#ifdef HAVE_SSL
-    if (acc->tls)
-    {
-	if ((e = smtp_tls_init(&srv, acc->tls_key_file, acc->tls_cert_file, 
-			acc->tls_trust_file, errstr)) != TLS_EOK)
-	{
-	    return exitcode_tls(e);
-	}
-    }
-#endif /* HAVE_SSL */
-
     /* connect */
     if ((e = smtp_connect(&srv, acc->host, acc->port, acc->timeout,
 		    NULL, NULL, errstr)) != NET_EOK)
@@ -474,8 +456,21 @@ int msmtp_rmqs(account_t *acc, int debug, const char *rmqs_argument,
 	return exitcode_net(e);
     }
 
+    /* prepare tls */
+#ifdef HAVE_TLS
+    if (acc->tls)
+    {
+	if ((e = smtp_tls_init(&srv, acc->tls_key_file, acc->tls_cert_file, 
+			acc->tls_trust_file, acc->tls_force_sslv3, errstr)) 
+		!= TLS_EOK)
+	{
+	    return exitcode_tls(e);
+	}
+    }
+#endif /* HAVE_TLS */
+
     /* start tls for ssmtp servers */
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
     if (acc->tls && acc->tls_nostarttls)
     {
 	if (debug)
@@ -498,7 +493,7 @@ int msmtp_rmqs(account_t *acc, int debug, const char *rmqs_argument,
 	    tls_cert_info_free(tci);
 	}
     }
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
 
     /* get greeting */
     if ((e = smtp_get_greeting(&srv, msg, NULL, errstr)) != SMTP_EOK)
@@ -515,7 +510,7 @@ int msmtp_rmqs(account_t *acc, int debug, const char *rmqs_argument,
     }
     
     /* start tls for starttls servers */
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
     if (acc->tls && !acc->tls_nostarttls)
     {
 	if (!(srv.cap.flags & SMTP_CAP_STARTTLS))
@@ -556,7 +551,7 @@ int msmtp_rmqs(account_t *acc, int debug, const char *rmqs_argument,
 	    return exitcode_smtp(e);
 	}
     }
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
 
     if (!(srv.cap.flags & SMTP_CAP_ETRN))
     {
@@ -615,9 +610,9 @@ int msmtp_serverinfo(account_t *acc, int debug, list_t **msg, char **errstr)
     char *server_address = NULL;
     char *server_greeting = NULL;
     int e;
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
     tls_cert_info_t *tci = NULL;
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
     
     *errstr = NULL;
     *msg = NULL;
@@ -625,20 +620,6 @@ int msmtp_serverinfo(account_t *acc, int debug, list_t **msg, char **errstr)
     /* create a new smtp_server_t */
     srv = smtp_new(debug ? stdout : NULL, acc->protocol);
 
-    /* prepare tls */
-#ifdef HAVE_SSL
-    if (acc->tls)
-    {
-	tci = tls_cert_info_new();
-	if ((e = smtp_tls_init(&srv, acc->tls_key_file, acc->tls_cert_file, 
-			acc->tls_trust_file, errstr)) != TLS_EOK)
-	{
-	    e = exitcode_tls(e);
-	    goto error_exit;
-	}
-    }
-#endif /* HAVE_SSL */
-
     /* connect */
     if ((e = smtp_connect(&srv, acc->host, acc->port, acc->timeout,
 		    &server_canonical_name, &server_address, errstr))
@@ -648,8 +629,23 @@ int msmtp_serverinfo(account_t *acc, int debug, list_t **msg, char **errstr)
        	goto error_exit;
     }
 
+    /* prepare tls */
+#ifdef HAVE_TLS
+    if (acc->tls)
+    {
+	tci = tls_cert_info_new();
+	if ((e = smtp_tls_init(&srv, acc->tls_key_file, acc->tls_cert_file, 
+			acc->tls_trust_file, acc->tls_force_sslv3, errstr)) 
+		!= TLS_EOK)
+	{
+	    e = exitcode_tls(e);
+	    goto error_exit;
+	}
+    }
+#endif /* HAVE_TLS */
+
     /* start tls for ssmtp servers */
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
     if (acc->tls && acc->tls_nostarttls)
     {
 	if ((e = smtp_tls(&srv, acc->host, acc->tls_nocertcheck, tci, errstr))
@@ -660,7 +656,7 @@ int msmtp_serverinfo(account_t *acc, int debug, list_t **msg, char **errstr)
 	    goto error_exit;
 	}
     }
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
 
     /* get greeting */
     if ((e = smtp_get_greeting(&srv, msg, &server_greeting, 
@@ -680,7 +676,7 @@ int msmtp_serverinfo(account_t *acc, int debug, list_t **msg, char **errstr)
     }
     
     /* start tls for starttls servers */
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
     if (acc->tls && !acc->tls_nostarttls)
     {
 	if (!(srv.cap.flags & SMTP_CAP_STARTTLS))
@@ -712,7 +708,7 @@ int msmtp_serverinfo(account_t *acc, int debug, list_t **msg, char **errstr)
 	    goto error_exit;
 	}
     }
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
 
     /* end session */
     msmtp_endsession(&srv, 1);
@@ -746,17 +742,17 @@ int msmtp_serverinfo(account_t *acc, int debug, list_t **msg, char **errstr)
     {
 	printf("    %s\n", msmtp_sanitize_string(server_greeting));
     }
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
     if (acc->tls)
     {
 	msmtp_print_tls_cert_info(tci);
     }
-#endif /* HAVE_SSL */
-#ifdef HAVE_SSL
+#endif /* HAVE_TLS */
+#ifdef HAVE_TLS
     if (srv.cap.flags == 0 && !(acc->tls && !acc->tls_nostarttls))
-#else /* not HAVE_SSL */
+#else /* not HAVE_TLS */
     if (srv.cap.flags == 0)
-#endif /* not HAVE_SSL */
+#endif /* not HAVE_TLS */
     {
 	printf(_("No special capabilities.\n"));
     }
@@ -801,12 +797,12 @@ int msmtp_serverinfo(account_t *acc, int debug, list_t **msg, char **errstr)
     	    printf("    DSN:\n        %s\n", _("Support for "
 			"Delivery Status Notifications"));
        	}
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
 	if ((acc->tls && !acc->tls_nostarttls) 
 		|| (srv.cap.flags & SMTP_CAP_STARTTLS))
-#else /* not HAVE_SSL */
+#else /* not HAVE_TLS */
         if (srv.cap.flags & SMTP_CAP_STARTTLS)
-#endif /* not HAVE_SSL */
+#endif /* not HAVE_TLS */
 	{
     	    printf("    STARTTLS:\n        %s\n", _("Support for "
 			"TLS encryption via the STARTTLS command"));
@@ -845,11 +841,11 @@ int msmtp_serverinfo(account_t *acc, int debug, list_t **msg, char **errstr)
     	    }
 	    printf("\n");
        	}
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
 	if ((srv.cap.flags & SMTP_CAP_STARTTLS) && !acc->tls)
-#else /* not HAVE_SSL */
+#else /* not HAVE_TLS */
 	if (srv.cap.flags & SMTP_CAP_STARTTLS)
-#endif /* not HAVE_SSL */
+#endif /* not HAVE_TLS */
 	{
     	    printf(_("This server might advertise more or other "
 		    "capabilities when TLS is active.\n"));
@@ -861,12 +857,12 @@ int msmtp_serverinfo(account_t *acc, int debug, list_t **msg, char **errstr)
 error_exit:
     free(server_canonical_name);
     free(server_address);
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
     if (tci)
     {
 	tls_cert_info_free(tci);
     }
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
     free(server_greeting);
     return e;
 }
@@ -1343,9 +1339,9 @@ int msmtp_sendmail(account_t *acc, list_t *recipients, int read_recipients,
     smtp_server_t srv;
     FILE *tmpfile = NULL;
     int e;
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
     tls_cert_info_t *tci = NULL;
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
     
     *errstr = NULL;
     *msg = NULL;
@@ -1372,17 +1368,18 @@ int msmtp_sendmail(account_t *acc, list_t *recipients, int read_recipients,
     srv = smtp_new(debug ? stdout : NULL, acc->protocol);
 
     /* prepare tls */
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
     if (acc->tls)
     {
 	if ((e = smtp_tls_init(&srv, acc->tls_key_file, acc->tls_cert_file, 
-			acc->tls_trust_file, errstr)) != TLS_EOK)
+			acc->tls_trust_file, acc->tls_force_sslv3, errstr)) 
+		!= TLS_EOK)
 	{
 	    e = exitcode_tls(e);
 	    goto error_exit;
 	}
     }
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
 
     /* connect */
     if ((e = smtp_connect(&srv, acc->host, acc->port, acc->timeout,
@@ -1393,7 +1390,7 @@ int msmtp_sendmail(account_t *acc, list_t *recipients, int read_recipients,
     }
 
     /* start tls for ssmtp servers */
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
     if (acc->tls && acc->tls_nostarttls)
     {
 	if (debug)
@@ -1417,7 +1414,7 @@ int msmtp_sendmail(account_t *acc, list_t *recipients, int read_recipients,
 	    tls_cert_info_free(tci);
 	}
     }
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
 
     /* get greeting */
     if ((e = smtp_get_greeting(&srv, msg, NULL, errstr)) != SMTP_EOK)
@@ -1436,7 +1433,7 @@ int msmtp_sendmail(account_t *acc, list_t *recipients, int read_recipients,
     }
 
     /* start tls for starttls servers */
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
     if (acc->tls && !acc->tls_nostarttls)
     {
 	if (!(srv.cap.flags & SMTP_CAP_STARTTLS))
@@ -1481,7 +1478,7 @@ int msmtp_sendmail(account_t *acc, list_t *recipients, int read_recipients,
 	    goto error_exit;
 	}
     }
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
 
     /* test for needed features */
     if ((acc->dsn_return || acc->dsn_notify) && !(srv.cap.flags & SMTP_CAP_DSN))
@@ -1812,19 +1809,20 @@ void msmtp_log_to_file(const char *logfile, const char *loginfo)
     FILE *f;
     time_t t;
     struct tm *tm;
-    const char *failure_reason;
+    char *failure_reason;
     char time_str[64];
-    
+    int e;
     
     /* get time */
     if ((t = time(NULL)) < 0)
     {
-	failure_reason = strerror(errno);
+	failure_reason = xasprintf(_("cannot get system time: %s"), 
+		strerror(errno));
 	goto log_failure;
     }
     if (!(tm = localtime(&t)))
     {
-	failure_reason = _("cannot convert UTC time to local time");
+	failure_reason = xstrdup(_("cannot convert UTC time to local time"));
 	goto log_failure;
     }
     (void)strftime(time_str, sizeof(time_str), "%b %d %H:%M:%S", tm);
@@ -1838,19 +1836,34 @@ void msmtp_log_to_file(const char *logfile, const char *loginfo)
     {
 	if (!(f = fopen(logfile, "a")))
 	{
-	    failure_reason = strerror(errno);
+	    failure_reason = xasprintf(_("cannot open: %s"), strerror(errno));
+	    goto log_failure;
+	}
+	if ((e = lock_file(f, TOOLS_LOCK_WRITE, 10)) != 0)
+	{
+	    if (e == 1)
+	    {
+		failure_reason = xasprintf(
+			_("cannot lock (tried for %d seconds): %s"), 
+			10, strerror(errno));
+	    }
+	    else
+	    {
+		failure_reason = xasprintf(_("cannot lock: %s"), 
+			strerror(errno));
+	    }
 	    goto log_failure;
 	}
     }
     if ((fputs(time_str, f) == EOF) || (fputc(' ', f) == EOF)
 	|| (fputs(loginfo, f) == EOF) || (fputc('\n', f) == EOF))
     {
-	failure_reason = _("output error");
+	failure_reason = xstrdup(_("output error"));
 	goto log_failure;
     }
     if (f != stdout && fclose(f) != 0)
     {
-	failure_reason = strerror(errno);
+	failure_reason = xstrdup(strerror(errno));
 	goto log_failure;
     }
 
@@ -1859,6 +1872,7 @@ void msmtp_log_to_file(const char *logfile, const char *loginfo)
     /* error exit target */
 log_failure:
     print_error(_("cannot log to %s: %s"), logfile, failure_reason);
+    free(failure_reason);
     if (loginfo)
     {
 	print_error(_("log info was: %s"), loginfo);
@@ -1968,29 +1982,26 @@ void msmtp_print_version(void)
     char *homedir;
     char *userconffile;
     
-    printf(_("%s version %s\nNLS: "), PACKAGE_NAME, VERSION);
-#ifdef ENABLE_NLS
-    printf(_("enabled, LOCALEDIR is %s\n"), LOCALEDIR);
-#else
-    printf("disabled\n");
-#endif
-    printf(_("TLS/SSL library: %s\n"
-		"Authentication library: %s\n"
-		"Supported authentication methods:\n"),
-#ifdef HAVE_GNUTLS
+    printf(_("%s version %s\n"), PACKAGE_NAME, VERSION);
+    /* TLS/SSL support */
+    printf(_("TLS/SSL library: %s\n"),
+#ifdef HAVE_LIBGNUTLS
 	    "GnuTLS"
 #elif defined (HAVE_OPENSSL)
-	    "OpenSSL"
+    	    "OpenSSL"
 #else
 	    _("none")
 #endif
-	    ,
-#ifdef USE_GSASL
-	    _("GNU SASL")
+	  );
+    /* Authentication support */
+    printf(_("Authentication library: %s\n"
+		"Supported authentication methods:\n"),
+#ifdef HAVE_LIBGSASL
+	    "GNU SASL"
 #else
 	    _("built-in")
-#endif /* USE_GSASL */
-	      );
+#endif /* HAVE_LIBGSASL */
+	  );
     if (smtp_client_supports_authmech("PLAIN"))
     {
 	printf("plain ");
@@ -2019,6 +2030,23 @@ void msmtp_print_version(void)
     {
 	printf("ntlm ");
     }
+    printf("\n");
+    /* Internationalized Domain Names support */
+    printf(_("IDN support: "));
+#ifdef HAVE_LIBIDN
+    printf(_("enabled"));
+#else
+    printf(_("disabled"));
+#endif
+    printf("\n");
+    /* Native language support */
+    printf(_("NLS: "));
+#ifdef ENABLE_NLS
+    printf(_("enabled"));
+    printf(_(", LOCALEDIR is %s"), LOCALEDIR);
+#else
+    printf(_("disabled"));
+#endif
     printf("\n");
     sysconfdir = get_sysconfdir();
     sysconffile = get_filename(sysconfdir, SYSCONFFILE);
@@ -2030,11 +2058,13 @@ void msmtp_print_version(void)
     printf(_("User configuration file name: %s\n"), userconffile);
     free(userconffile);
     free(homedir);
-    printf(_("\nCopyright (C) 2004, 2005 Martin Lambers and others.\n"
-    		"This is free software; see the source for copying "
-		"conditions.  There is NO\n"
-		"warranty; not even for MERCHANTABILITY or FITNESS FOR A "
-		"PARTICULAR PURPOSE.\n"));
+    printf("\n");
+    printf(_("Copyright (C) 2006 Martin Lambers and others.\n"
+		"This is free software.  You may redistribute copies of "
+		    "it under the terms of\n"
+		"the GNU General Public License "
+		    "<http://www.gnu.org/licenses/gpl.html>.\n"
+		"There is NO WARRANTY, to the extent permitted by law.\n"));
 }
 
 
@@ -2057,66 +2087,69 @@ void msmtp_print_help(void)
 	    "  Print information about a server.\n"
 	    "Remote Message Queue Starting mode:\n"
 	    "  %s [option...] --rmqs=host|@domain|#queue\n"
-	    "  Send a Remote Message Queue Starting request to a server\n"
+	    "  Send a Remote Message Queue Starting request to a server.\n"
             "\nOPTIONS:\n\n"
 	    "General options:\n"
-	    "  --version                    print version\n"
-	    "  --help                       print help\n"
-	    "  -P, --pretend                print configuration info and exit\n"
-	    "  -d, --debug                  print debugging information\n"
+	    "  --version                    Print version.\n"
+	    "  --help                       Print help.\n"
+	    "  -P, --pretend                Print configuration info and "
+	        "exit.\n"
+	    "  -d, --debug                  Print debugging information.\n"
 	    "Changing the mode of operation:\n"
-	    "  -S, --serverinfo             print information about the "
-	    	"server\n"
-	    "  --rmqs=host|@domain|#queue   send a Remote Message Queue "
-	    	"Starting request\n"
+	    "  -S, --serverinfo             Print information about the "
+	    	"server.\n"
+	    "  --rmqs=host|@domain|#queue   Send a Remote Message Queue "
+	    	"Starting request.\n"
             "Configuration options:\n"
-	    "  -C, --file=filename          set configuration file\n"
-	    "  -a, --account=id             use the given account instead of "
+	    "  -C, --file=filename          Set configuration file.\n"
+	    "  -a, --account=id             Use the given account instead of "
 	        "the account\n"
 	    "                               named \"default\"; its settings "
 	        "may be changed\n"
-	    "                               with command line options\n"
-	    "  --host=hostname              set the server, use only command "
+	    "                               with command line options.\n"
+	    "  --host=hostname              Set the server, use only command "
 	    	"line settings;\n"
 	    "                               do not use any configuration file "
-	    	"data\n"
-	    "  --port=number                set port number\n"
-	    "  --timeout=(off|seconds)      set/unset network timeout in "
-		"seconds\n"
-	    "  --protocol=(smtp|lmtp)       use the given sub protocol\n"
-	    "  --domain=string              set the argument of EHLO or LHLO "
-	    	"command\n"
-	    "  --auth[=(on|off|method)]     enable/disable authentication and "
+	    	"data.\n"
+	    "  --port=number                Set port number.\n"
+	    "  --timeout=(off|seconds)      Set/unset network timeout in "
+		"seconds.\n"
+	    "  --protocol=(smtp|lmtp)       Use the given sub protocol.\n"
+	    "  --domain=string              Set the argument of EHLO or LHLO "
+	    	"command.\n"
+	    "  --auth[=(on|off|method)]     Enable/disable authentication and "
 	    	"optionally\n"
-	    "                               choose the method\n"
-	    "  --user=[username]            set/unset user name for "
-	    	"authentication\n"
-	    "  --tls[=(on|off)]             enable/disable TLS encryption\n"
-	    "  --tls-trust-file=[file]      set/unset trust file for TLS\n"
-            "  --tls-key-file=[file]        set/unset private key file for "
-	    	"TLS\n"
-	    "  --tls-cert-file=[file]       set/unset private cert file for "
-	    	"TLS\n"
-	    "  --tls-certcheck[=(on|off)]   enable/disable server certificate "
-	    	"checks for TLS\n"
-	    "  --tls-starttls[=(on|off)]    enable/disable STARTTLS for TLS\n"
+	    "                               choose the method.\n"
+	    "  --user=[username]            Set/unset user name for "
+	    	"authentication.\n"
+	    "  --tls[=(on|off)]             Enable/disable TLS encryption.\n"
+	    "  --tls-starttls[=(on|off)]    Enable/disable STARTTLS for TLS.\n"
+	    "  --tls-trust-file=[file]      Set/unset trust file for TLS.\n"
+            "  --tls-key-file=[file]        Set/unset private key file for "
+	    	"TLS.\n"
+	    "  --tls-cert-file=[file]       Set/unset private cert file for "
+	    	"TLS.\n"
+	    "  --tls-certcheck[=(on|off)]   Enable/disable server certificate "
+	    	"checks for TLS.\n"
+	    "  --tls-force-sslv3[=(on|off)] Enable/disable restriction to "
+	        "SSLv3.\n"
             "Options specific to sendmail mode:\n"
-	    "  --auto-from[=(on|off)]       enable/disable automatic "
-	        "envelope-from addresses\n"
-	    "  -f, --from=address           set envelope from address\n"
-	    "  --maildomain=[domain]        set the domain for automatic "
+	    "  --auto-from[=(on|off)]       Enable/disable automatic "
+	        "envelope-from addresses.\n"
+	    "  -f, --from=address           Set envelope from address.\n"
+	    "  --maildomain=[domain]        Set the domain for automatic "
 	        "envelope from\n"
-	    "                               addresses\n"
-	    "  -N, --dsn-notify=(off|cond)  set/unset DSN conditions\n"
-	    "  -R, --dsn-return=(off|ret)   set/unset DSN amount\n"
-	    "  --keepbcc[=(on|off)]         enable/disable preservation of the "
-	    	"Bcc header\n"
-            "  -X, --logfile=[file]         set/unset log file\n"
-	    "  --syslog[=(on|off|facility)] enable/disable/configure syslog "
-	    	"logging\n"
-	    "  -t, --read-recipients        read additional recipients from "
-	    	"the mail\n"
-	    "  --                           end of options\n"
+	    "                               addresses.\n"
+	    "  -N, --dsn-notify=(off|cond)  Set/unset DSN conditions.\n"
+	    "  -R, --dsn-return=(off|ret)   Set/unset DSN amount.\n"
+	    "  --keepbcc[=(on|off)]         Enable/disable preservation of the "
+	    	"Bcc header.\n"
+            "  -X, --logfile=[file]         Set/unset log file.\n"
+	    "  --syslog[=(on|off|facility)] Enable/disable/configure syslog "
+	    	"logging.\n"
+	    "  -t, --read-recipients        Read additional recipients from "
+	    	"the mail.\n"
+	    "  --                           End of options.\n"
             "Accepted but ignored: -A, -B, -bm, -F, -G, -h, -i, -L, -m, -n, "
 	    	"-O, -o, -v\n"
 	    "\nReport bugs to <%s>.\n"),
@@ -2161,18 +2194,19 @@ typedef struct
 #define LONGONLYOPT_AUTH		5
 #define LONGONLYOPT_USER		6
 #define LONGONLYOPT_TLS			7
-#define LONGONLYOPT_TLS_TRUST_FILE	8
-#define LONGONLYOPT_TLS_KEY_FILE	9
-#define LONGONLYOPT_TLS_CERT_FILE	10
-#define LONGONLYOPT_TLS_CERTCHECK	11
-#define LONGONLYOPT_TLS_STARTTLS	12
-#define LONGONLYOPT_PROTOCOL		13
-#define LONGONLYOPT_DOMAIN		14
-#define LONGONLYOPT_KEEPBCC		15
-#define LONGONLYOPT_RMQS		16
-#define LONGONLYOPT_SYSLOG		17
-#define LONGONLYOPT_MAILDOMAIN		18
-#define LONGONLYOPT_AUTO_FROM		19
+#define LONGONLYOPT_TLS_STARTTLS	8
+#define LONGONLYOPT_TLS_TRUST_FILE	9
+#define LONGONLYOPT_TLS_KEY_FILE	10
+#define LONGONLYOPT_TLS_CERT_FILE	11
+#define LONGONLYOPT_TLS_CERTCHECK	12
+#define LONGONLYOPT_TLS_FORCE_SSLV3	13
+#define LONGONLYOPT_PROTOCOL		14
+#define LONGONLYOPT_DOMAIN		15
+#define LONGONLYOPT_KEEPBCC		16
+#define LONGONLYOPT_RMQS		17
+#define LONGONLYOPT_SYSLOG		18
+#define LONGONLYOPT_MAILDOMAIN		19
+#define LONGONLYOPT_AUTO_FROM		20
 
 int msmtp_cmdline(msmtp_cmdline_conf_t *conf, int argc, char *argv[])
 {
@@ -2198,11 +2232,13 @@ int msmtp_cmdline(msmtp_cmdline_conf_t *conf, int argc, char *argv[])
 	{ "auth",            optional_argument, 0, LONGONLYOPT_AUTH },
 	{ "user",            required_argument, 0, LONGONLYOPT_USER },
 	{ "tls",             optional_argument, 0, LONGONLYOPT_TLS },
+	{ "tls-starttls",    optional_argument, 0, LONGONLYOPT_TLS_STARTTLS },
 	{ "tls-trust-file",  required_argument, 0, LONGONLYOPT_TLS_TRUST_FILE },
 	{ "tls-key-file",    required_argument, 0, LONGONLYOPT_TLS_KEY_FILE },
 	{ "tls-cert-file",   required_argument, 0, LONGONLYOPT_TLS_CERT_FILE },
 	{ "tls-certcheck",   optional_argument, 0, LONGONLYOPT_TLS_CERTCHECK },
-	{ "tls-starttls",    optional_argument, 0, LONGONLYOPT_TLS_STARTTLS },
+	{ "tls-force-sslv3", optional_argument, 0, 
+	    LONGONLYOPT_TLS_FORCE_SSLV3 },
 	{ "dsn-notify",      required_argument, 0, 'N' },
 	{ "dsn-return",      required_argument, 0, 'R' },
 	{ "protocol",        required_argument, 0, LONGONLYOPT_PROTOCOL },
@@ -2420,6 +2456,7 @@ int msmtp_cmdline(msmtp_cmdline_conf_t *conf, int argc, char *argv[])
 		}
 		else
 		{
+		    conf->cmdline_account->auth_mech = NULL;
 		    print_error(_("invalid argument %s for %s"), 
 			    optarg, "--auth");
 		    error_code = 1;
@@ -2451,6 +2488,24 @@ int msmtp_cmdline(msmtp_cmdline_conf_t *conf, int argc, char *argv[])
 		}
 		conf->cmdline_account->mask |= ACC_TLS;
 		break;
+		
+	    case LONGONLYOPT_TLS_STARTTLS:
+	    	if (!optarg || is_on(optarg))
+    		{
+		    conf->cmdline_account->tls_nostarttls = 0;
+		}
+		else if (is_off(optarg))
+		{
+		    conf->cmdline_account->tls_nostarttls = 1;
+		}
+		else
+		{
+		    print_error(_("invalid argument %s for %s"), 
+			    optarg, "--tls-starttls");
+		    error_code = 1;
+		}
+		conf->cmdline_account->mask |= ACC_TLS_NOSTARTTLS;
+		break;
 
 	    case LONGONLYOPT_TLS_TRUST_FILE:
 		free(conf->cmdline_account->tls_trust_file);
@@ -2510,22 +2565,22 @@ int msmtp_cmdline(msmtp_cmdline_conf_t *conf, int argc, char *argv[])
 		conf->cmdline_account->mask |= ACC_TLS_NOCERTCHECK;
 		break;
 
-	    case LONGONLYOPT_TLS_STARTTLS:
-	    	if (!optarg || is_on(optarg))
+	    case LONGONLYOPT_TLS_FORCE_SSLV3:
+	   	if (!optarg || is_on(optarg))
     		{
-		    conf->cmdline_account->tls_nostarttls = 0;
+		    conf->cmdline_account->tls_force_sslv3 = 1;
 		}
 		else if (is_off(optarg))
 		{
-		    conf->cmdline_account->tls_nostarttls = 1;
+		    conf->cmdline_account->tls_force_sslv3 = 0;
 		}
 		else
 		{
 		    print_error(_("invalid argument %s for %s"), 
-			    optarg, "--tls-starttls");
+			    optarg, "--tls-force-sslv3");
 		    error_code = 1;
 		}
-		conf->cmdline_account->mask |= ACC_TLS_NOSTARTTLS;
+		conf->cmdline_account->mask |= ACC_TLS_FORCE_SSLV3;
 		break;
 
 	    case 'N':
@@ -2913,20 +2968,22 @@ void msmtp_print_conf(msmtp_cmdline_conf_t conf, account_t *account)
 	    "password        = %s\n"
 	    "ntlmdomain      = %s\n"
 	    "tls             = %s\n"
+	    "tls_starttls    = %s\n"
 	    "tls_trust_file  = %s\n"
 	    "tls_key_file    = %s\n"
 	    "tls_cert_file   = %s\n"
-	    "tls_starttls    = %s\n"
-	    "tls_certcheck   = %s\n",
+	    "tls_certcheck   = %s\n"
+	    "tls_force_sslv3 = %s\n",
 	    account->username ? account->username : _("(not set)"),
 	    account->password ? "*" : _("(not set)"),
 	    account->ntlmdomain ? account->ntlmdomain : _("(not set)"),
 	    account->tls ? _("on") : _("off"), 
+	    account->tls_nostarttls ? _("off") : _("on"),
 	    account->tls_trust_file ? account->tls_trust_file : _("(not set)"),
 	    account->tls_key_file ? account->tls_key_file : _("(not set)"),
 	    account->tls_cert_file ? account->tls_cert_file : _("(not set)"),
-	    account->tls_nostarttls ? _("off") : _("on"),
-	    account->tls_nocertcheck ? _("off") : _("on"));
+	    account->tls_nocertcheck ? _("off") : _("on"),
+	    account->tls_force_sslv3 ? _("on") : _("off"));
     if (conf.sendmail)
     {
 	printf("auto_from       = %s\n"
@@ -2983,7 +3040,7 @@ int main(int argc, char *argv[])
     int e;
     list_t *lp;
     /* misc */
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
     int tls_lib_initialized = 0;
 #endif
     int net_lib_initialized = 0;
@@ -3000,6 +3057,15 @@ int main(int argc, char *argv[])
     struct servent *se;    
 
     
+    /* Avoid the side effects of text mode interpretations on DOS systems. */
+#ifdef W32_NATIVE
+    _setmode(_fileno(stdin), _O_BINARY);
+    _fmode = _O_BINARY;
+#elif defined DJGPP
+    setmode(fileno(stdin), O_BINARY);
+    _fmode = O_BINARY;
+#endif
+
     errstr = NULL;
     errmsg = NULL;
     
@@ -3126,10 +3192,13 @@ int main(int argc, char *argv[])
 		account->port = se ? ntohs(se->s_port) : 25;
 	    }
 	}
-	else /* LMTP */
+	else /* LMTP. Has no default port as of 2006-06-17. */
 	{
 	    se = getservbyname("lmtp", NULL);
-	    account->port = se ? ntohs(se->s_port) : 2003;
+	    if (se)
+	    {
+		account->port = ntohs(se->s_port);
+	    }
 	}
     }
     if (conf.sendmail && account->auto_from)
@@ -3194,7 +3263,7 @@ int main(int argc, char *argv[])
     net_lib_initialized = 1;
     if (account->tls)
     {
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
 	if ((e = tls_lib_init(&errstr)) != TLS_EOK)
 	{
 	    print_error(_("cannot initialize TLS library: %s"), 
@@ -3203,11 +3272,11 @@ int main(int argc, char *argv[])
 	    goto exit;
 	}
 	tls_lib_initialized = 1;
-#else /* not HAVE_SSL */
+#else /* not HAVE_TLS */
 	print_error(_("support for TLS is not compiled in"));
 	error_code = EX_UNAVAILABLE;
 	goto exit;
-#endif /* not HAVE_SSL */
+#endif /* not HAVE_TLS */
     }
 
     /* do the work */
@@ -3356,12 +3425,12 @@ exit:
     /* clean up */
     free(loaded_system_conffile);
     free(loaded_user_conffile);
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
     if (tls_lib_initialized) 
     {
 	tls_lib_deinit();
     }
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
     if (net_lib_initialized)
     {
 	net_lib_deinit();
